@@ -4,7 +4,7 @@ use strict;
 use AI::DecisionTree::Instance;
 use Carp;
 use vars qw($VERSION @ISA);
-$VERSION = '0.08';
+$VERSION = '0.09';
 
 
 sub new {
@@ -73,7 +73,8 @@ sub train {
   local $self->{max_depth} = $args{max_depth} if exists $args{max_depth};
   $self->{depth} = 0;
   $self->{tree} = $self->_expand_node( instances => $self->{instances} );
-
+  $self->{total_instances} = @{$self->{instances}};
+  
   $self->prune_tree if $self->{prune};
   $self->do_purge if $self->purge;
   return 1;
@@ -135,6 +136,10 @@ sub _expand_node {
   my @results = map {$_,$results{$_}} sort {$results{$b} <=> $results{$a}} keys %results;
   my %node = ( distribution => \@results, instances => scalar @$instances );
 
+  foreach (keys %results) {
+    $self->{prior_freqs}{$_} += $results{$_};
+  }
+
   if (keys(%results) == 1) {
     # All these instances have the same result - make this node a leaf
     $node{result} = $self->_result($instances->[0]);
@@ -302,22 +307,30 @@ sub _traverse {
 
 sub get_result {
   my ($self, %args) = @_;
-  croak "Missing 'attributes' parameter" unless $args{attributes};
-  my $a = $args{attributes};
+  croak "Missing 'attributes' or 'callback' parameter" unless $args{attributes} or $args{callback};
 
   $self->train unless $self->{tree};
   my $tree = $self->{tree};
-  my $count = 0;
   
   while (1) {
     if (exists $tree->{result}) {
       my $r = $tree->{result};
-      return ($r, $tree->{distribution}[1] / $tree->{instances}, $count) if wantarray;
-      return $r;
-    }
-    $count++;
+      return $r unless wantarray;
 
-    my $instance_val = exists $a->{$tree->{split_on}} ? $a->{$tree->{split_on}} : '<undef>';
+      my %dist = @{$tree->{distribution}};
+      my $confidence = $tree->{distribution}[1] / $tree->{instances};
+
+#      my $confidence = P(H|D) = [P(D|H)P(H)]/[P(D|H)P(H)+P(D|H')P(H')]
+#                              = [P(D|H)P(H)]/P(D);
+#      my $confidence = 
+#      $confidence *= $self->{prior_freqs}{$r} / $self->{total_instances};
+      
+      return ($r, $confidence, \%dist);
+    }
+    
+    my $instance_val = (exists $args{callback} ? $args{callback}->($tree->{split_on}) :
+			exists $args{attributes}{$tree->{split_on}} ? $args{attributes}{$tree->{split_on}} :
+			'<undef>');
     $tree = $tree->{children}{ $instance_val }
       or return undef;
   }
@@ -327,7 +340,7 @@ sub as_graphviz {
   my ($self, %args) = @_;
   my $colors = delete $args{leaf_colors} || {};
   require GraphViz;
-  my $g = GraphViz->new(@_);
+  my $g = GraphViz->new(%args);
 
   my $id = 1;
   my $add_edge = sub {
